#You provide primal (s_prim) and dual (s_dual) scaling coefficients, and the length of a discrete signal (n_f is half the length).

#This program computes T and T˜, the original biorthogonal but non-banded wavelet transforms, then computes V and V˜, transformation matrices used to band T, and then computes Q and Q˜, biorthogonal banded matrices corresponding to T and T˜. Note that it is possible to modify T and T˜ further via multiplication on the left by other biorthogonal matrices U and U˜ but this is not explored here.

#A scatter-plot of the cascade algorithm applied to scaling and/or wavelet functions generated by Q and Q˜ can be provided.

# V_Transforms Produces V and \tilde{V}.
# Input:
#    n_f - Half the length of the data    
#    s_prim, s_dual - Two (even-length) lists of biorthogonal scaling coefficients.
#    (Make sure that s_dual is the longer of the two!)
# Output:
#    V_prim, V_dual - The right-hand matrix used in order to make a simple modification of T_dual and T_prim

def V_Transforms(n_f,s_prim,s_dual):
    n_s=int((len(s_prim)-2)/2)        # The coefficients are s_0,... s_(2*n_s+1).
    w_prim=[]
    w_dual=[]
    odd=len(s_prim)-1
    for k in range(len(s_prim)):
        w_prim.append((-1)^k*s_dual[odd-k])
        w_dual.append((-1)^k*s_prim[odd-k])
        
    def Create_block_matrix_A(s):
        A=Matrix(QQ,n_s,2*n_s)
        for j in range(n_s):
            for k in range(2*n_s):
                if k-2*j >= 0:
                    A[j,k]=s[k-2*j]
                elif k-2*j < 0:
                    A[j,k]=0           
        return A;

    def Create_block_matrix_B(s):
        B=Matrix(QQ,n_s,2*n_s)
        for j in range(n_s):
            for k in range(2*n_s):
                if 2*n_s+k-2*j <= 2*n_s+1:
                    B[j,k]=s[2*n_s+(k-2*j)]
                elif 2*n_s+k-2*j > 2*n_s+1:
                    B[j,k]=0     
        return B;              
    
    S_Prim_a=Create_block_matrix_A(s_prim)
    S_Prim_b=Create_block_matrix_B(s_prim)
    S_Dual_a=Create_block_matrix_A(s_dual)
    S_Dual_b=Create_block_matrix_B(s_dual)
    
    W_Prim_a=Create_block_matrix_A(w_prim)
    W_Prim_b=Create_block_matrix_B(w_prim)
    W_Dual_a=Create_block_matrix_A(w_dual)
    W_Dual_b=Create_block_matrix_B(w_dual)
    
    Ident=identity_matrix(QQ,2*(n_f-n_s))
    Z_top=zero_matrix(QQ,2*n_s,2*(n_f-n_s))
    Z_side=zero_matrix(QQ,2*(n_f-n_s),n_s)
    
    R_a = ~(S_Dual_a*W_Prim_a.transpose())
    R_b = ~(S_Dual_b*W_Prim_b.transpose())
    V_dual=block_matrix([(R_a*S_Dual_a).transpose(), Z_top, (R_b*S_Dual_b).transpose(), Z_side, Ident, Z_side],nrows=2)
    V_prim=block_matrix([W_Prim_a.transpose(), Z_top, W_Prim_b.transpose(), Z_side, Ident, Z_side],nrows=2)

    return V_prim,V_dual
    
#T_Transforms produces T and \tilde{T}.
# Input:
#    n_f - Half the length of the data    
#    s_prim, s_dual - Two (even-length) lists of biorthogonal scaling coefficients.
#    (Make sure that s_dual is the longer of the two!)
# Output:
#    T_prim, T_dual - the two periodized transformation matrices.    
def T_Transforms(n_f,s_prim,s_dual):
    n_s=int((len(s_prim)-2)/2)        # The coefficients are s_0,... s_(2*n_s+1).
    w_prim=[]
    w_dual=[]
    odd=len(s_prim)-1
    for k in range(len(s_prim)):
        w_prim.append((-1)^k*s_dual[odd-k])
        w_dual.append((-1)^k*s_prim[odd-k])
        
    def Create_T_Matrix(n_f,n_s,s,w):
        T=Matrix(QQ,2*n_f,2*n_f)
        
        for j in range(2*n_f):
            for k in range(2*n_f):
                if is_even(j):    #Rows 0, 2, 4, etc. are coming from s.
                    N=int(j/2)
                    Q=N-(n_f-n_s)
                    if k-2*N >= 0 and k-2*N < len(s):
                        T[j,k]=s[k-2*N]
                    else:
                        T[j,k]=0
                                                    # This is the periodized block in the lower left.
                    if Q >= 0 and 2*n_s+k-2*Q >=0 and 2*n_s+k-2*Q <= 2*n_s+1:
                        T[j,k]=s[2*n_s+k-2*Q]

                elif is_odd(j):   #Rows 1, 3, 5, etc. are from w.
                    N=int((j-1)/2)
                    if k-2*N >= 0 and k-2*N < len(w):
                        T[j,k]=w[k-2*N]
                    else:
                        T[j,k]=0
                                                   # This is the periodized block in the lower left.
                    if Q >= 0 and 2*n_s+k-2*Q >=0 and 2*n_s+k-2*Q <= 2*n_s+1:
                        T[j,k]=w[2*n_s+k-2*Q]
        return T
   
    T_prim=Create_T_Matrix(n_f,n_s,s_prim,w_prim)
    T_dual=Create_T_Matrix(n_f,n_s,s_dual,w_dual)
    return T_prim, T_dual
    
#Perform the cascade algorithm, phi^(m+1)(x)=sum_k s_k phi^(m)(2x-k)
def Cascade(s):
    D=100                    #Detail - The reciprocal of the step size.
    Iter=10                   #Number of iterations
    Type='Haar'                 #Hat is good for symmetric phi, Haar in general.
                                #Characteristic on [0,1) = 'Haar'
                                #Hat function on [0,2] = 'Hat'
                                    
    Supp=len(s)-1             #Support length of phi
    X=[0..2*Supp,step=1/D]    #The interval
    P=Matrix(QQ,Iter,len(X))     #The matrix of approximation values
    for n in range(Iter):
        for k in range(len(X)):
            P[n,k]=0
            
    if Type=='Haar':
        for k in range(D):
            P[0,k]=1

    elif Type=='Hat':
        for k in range(D+1):        #from 0 to 1
            P[0,k]=N(k)
        for k in range(D+2,2*D+1):  #from 1 to 2
            P[0,k]=2-N(k)
    
    #This is where the magic happens.
    #TypeError: no conversion of this rational to integer...
    #I floated that term, and defined P over QQ.

    for n in range(Iter-1):
        for k in range((len(X)-1)/2):
            for m in range(len(s)):
                Entry=2*k-D*m-1
                if (Entry >= 1) and (Entry < ((len(X)-1))/2):
                    P[n+1,k]=P[n+1,k]+float(s[m]*P[n,Entry])
                    #P[n+1,k]+=(s[m]*P[n,2*k-D*(m-1)-1])                    
    return P[Iter-1,:]
    
#Run the cascade algorithm on Q prim and dual.
def MRA_fns(n_f, Q_prim, Q_dual):

    m_prim=[]
    n_prim=[]
    m_dual=[]
    n_dual=[]
    for k in range(n_f):
        m_prim.append(Q_prim[2*k])
        n_prim.append(Q_prim[2*k+1])
        m_dual.append(Q_dual[2*k])
        n_dual.append(Q_dual[2*k+1])
        
    m_prim_fns=[]
    n_prim_fns=[]
    m_dual_fns=[]
    n_dual_fns=[]
    for k in range(n_f):
        m_prim_fns.append(Cascade(m_prim[k]))
        n_prim_fns.append(Cascade(n_prim[k]))
        m_dual_fns.append(Cascade(m_dual[k]))
        n_dual_fns.append(Cascade(n_dual[k]))
    
    return m_prim_fns, n_prim_fns, m_dual_fns, n_dual_fns
    
#Sketch plots of the functions
#Currently set to plot the wavelet functions in the primal MRA.

def Plot_MRA(n_f,m_prim_fns, n_prim_fns, m_dual_fns, n_dual_fns):
    K=[]
    for k in range(n_f):
        K.append(list_plot(n_prim_fns[k][0],pointsize=1))
        K[k].show()
        
n_f=5
#s_prim=[0,2,0,0]
#s_dual=[1/2,1,1/2,0]
s_prim=[0,1/2,1,1/2,0,0]
s_dual=[-1/6, 1/3, 3/2, 2/3, -1/3,0]
V_prim, V_dual = V_Transforms(n_f,s_prim,s_dual)
T_prim,T_dual = T_Transforms(n_f,s_prim,s_dual)
Q_prim=(1/sqrt(2))*T_prim*V_dual
Q_dual=(1/sqrt(2))*T_dual*V_prim

m_prim_fns, n_prim_fns, m_dual_fns, n_dual_fns = MRA_fns(n_f, Q_prim, Q_dual)

Plot_MRA(n_f,m_prim_fns, n_prim_fns, m_dual_fns, n_dual_fns)
