% David Ferrone, January 2012

% This approximates the LEFT-HAND boundary function.

% M is the modified scaling coefficients
% N is the modified wavelet coefficients
% s is the scaling coefficients, w the wavelet coefficients
% L = length(x), the length of the interval.

function [p, q] = Boundary_Cascade(M, N, s, w, n_s, n_f, L)

% These are block matrices used in the refinement eqn.
A_0 = zeros(n_s,n_s);
A_1 = zeros(n_s,n_s);
A_2 = zeros(n_s,n_s);

% Block matrices in N used to define the wavelets.
C_0 = zeros(n_s,n_s);
C_1 = zeros(n_s,n_s);
C_2 = zeros(n_s,n_s);

for j = 1:n_s
    for k = 1:n_s
        A_0(j,k) = M(j,k);
        A_1(j,k) = M(j,k+n_s);
        A_2(j,k) = M(j,k+2*n_s);
        C_0(j,k) = N(j,k);
        C_1(j,k) = N(j,k+n_s);
        C_2(j,k) = N(j,k+2*n_s);        
    end;
end;

A = horzcat(A_1, A_2);
C = horzcat(C_1, C_2);

%%%%%%%%%%%%%%%%%%%%
% phi is a column vector of length 2*n_s representing interior scaling
% functions in the space V_1.
interior_scaling = Cascade(s);
phi = zeros(50,2*n_s,L);

for n=1:50
    for j=0:2*n_s-1
        for k=1:L
            if (k-1000*j >= 1) && (k-1000*j < (L-1))    %Staying in the domain of interior_scaling.
                phi(n,j+1,k) = interior_scaling(n, k-1000*j);
            end;
        end;
    end;
end;

phi_full = zeros(2*n_s+2,L);

for j = 0:2*n_s+1               %phi_full will be used to define the wavelet later, it has all 2n+1 shifts of phi(x).
    for k = 1:L
        if (k-1000*j >= 1) && (k-1000*j < (L-1))
            phi_full(j+1,k) = interior_scaling(50,k-1000*j);
        end;
    end;
end;

p_iter = zeros(50, n_s, L);     % This will approximate the n_s left-hand boundary functions.

for k=1:1001
    for j = 1:n_s
        p_iter(1, j, k+1000*(j-1)) = 1;     % Setting the initial condition
    end;
end;

for j = 1:n_s
    for n = 1:49
        for k = 1:L/2
            p_iter(n+1,j,k) = sqrt(2)*((A_0(j,:)*p_iter(n,:,2*k)') + (A(j,:)*phi(n,:,2*k)'));   % The refinement eqn.
        end;
    end;
end;

p_0 = zeros(n_s,L);     % p_0 will be *just* the left-hand boundary scaling functions in our MRA.
p = zeros(n_f,L);       % p are the scaling functions, here we will have defined the left-hand and interior.

for j=1:n_s
    for k=1:L
        p(j,k) = p_iter(50,j,k);
        p_0(j,k) = p(j,k);
        if (j+n_s <= n_f-n_s)
            p(j+n_s,k) = phi(50, j+n_s, k);     % Interior scaling functions, support should begin at n_s.
        end;
    end;
end;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



% for j=1:n_s
%     for k = 1:L
%         phi_0(j,k) = phi(50,j,k);
%         phi_1(j,k) = phi(50,j+n_s,k);
%     end;
% end;

q = zeros(n_f,L);       % q are the wavelet functions.

for j = 1:n_s
        for k = 1:L/2
            q(j,k) = C_0(j,:)*p_0(:,2*k) + C(j,:)*phi(50,:,2*k)';
            %+ C_1(j,:)*phi_0(:,2*k) + C_2(j,:)*phi_1(:,2*k);   % left-hand boundary wavelet eqn
        end;
end;

% for j = n_s+1:n_f-n_s
%     for k = 1:L/2
%         q(j,k) = w(:)'*phi_full(:,2*k)
%     end;
% end;

end