% David Ferrone, January 2012

% Create T Matrix
%
% Input: n_f (half the length of the signal)
%        n_s (for scaling coeffs s_0,...,s_{2n_s+1})
%        s (a vector of scaling coefficients)
%        w (a vector of wavelet coefficients)
%
% Output: A (periodized) matrix T representing the discrete wavelet
% transform applied to a periodic signal of length 2*n_f.

function T = Create_T_Matrix(n_f, n_s, s, w)
T=zeros(2*n_f,2*n_f);

for j=0:2*n_f-1
    for k=0:2*n_f-1
        if (mod(j,2)==0)    %Rows 0, 2, 4, etc. are coming from s.
            N=j/2;
            Q=N-(n_f-n_s);
            
            if (k-2*N >= 0) && (k-2*N < length(s))
                T(j+1,k+1)=s(k-2*N+1);                  %added one... works?
            else
                T(j+1,k+1)=0;
            end;
                       % This is the periodized block in the bottom left.
            if (Q >= 0) && (2*n_s+k-2*Q >=0) && (2*n_s+k-2*Q <= 2*n_s+1)
                T(j+1,k+1)=s(2*n_s+k-2*Q+1);            %added one...
            end;

        elseif (mod(j,2)==1)   %Rows 1, 3, 5, etc. are from w.
                    N=(j-1)/2;
                    if (k-2*N >= 0) && (k-2*N < length(w))
                        T(j+1,k+1)=w(k-2*N+1);      %added one.
                    else
                        T(j+1,k+1)=0;
                    end;
                       % This is the periodized block in the bottom left.
                    if (Q >= 0) && (2*n_s+k-2*Q >=0) && (2*n_s+k-2*Q <= 2*n_s+1)
                        T(j+1,k+1)=w(2*n_s+k-2*Q+1);        %added one.
                    end;
        end;
    end;
end;
end